using System;
using System.Configuration;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Security.Cryptography;
using System.Threading;

using KnownObjects;
using Belikov.GenuineChannels;
using Belikov.GenuineChannels.BroadcastEngine;
using Belikov.GenuineChannels.DotNetRemotingLayer;
using Belikov.GenuineChannels.GenuineUdp;
using Belikov.GenuineChannels.Logbook;
using Belikov.GenuineChannels.Security;
using Belikov.GenuineChannels.TransportContext;

namespace Client
{
	/// <summary>
	/// Client.
	/// </summary>
	public class Client
	{
		/// <summary>
		/// Global receiver instance.
		/// </summary>
		public static MessageReceiver MessageReceiver = new MessageReceiver();

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			// wait for the server
			Console.WriteLine("Sleep for 3 seconds.");
			Thread.Sleep(TimeSpan.FromSeconds(3));

			// setup .NET Remoting
			Console.WriteLine("Configuring Remoting environment...");
			System.Configuration.ConfigurationSettings.GetConfig("DNS");
			GenuineGlobalEventProvider.GenuineChannelsGlobalEvent += new GenuineChannelsGlobalEventHandler(GenuineChannelsEventHandler);

			try
			{
				RemotingConfiguration.Configure("Client.exe.config");
				//GlobalLoggerContainer.Logger = new BinaryLog(@"c:\tmp\client.log");

				Console.WriteLine(".NET Remoting has been configured from Client.exe.config file.");

				// Set up Security Session
				IExchangeKeyProvider iExchangeKeyProvider = (IExchangeKeyProvider) Activator.GetObject(typeof(IExchangeKeyProvider),
					ConfigurationSettings.AppSettings["RemoteHostUri"] + "/KeyProvider.rem");
				SymmetricAlgorithm symmetricAlgorithm = SymmetricAlgorithm.Create();
				symmetricAlgorithm.Key = iExchangeKeyProvider.GetKey();
				symmetricAlgorithm.Mode = CipherMode.ECB;
				iExchangeKeyProvider.GetKey();

				// declare key provider, it will be used automatically for decryption received messages
				KeyProvider_KnownSymmetric keyProvider_KnownSymmetric = new KeyProvider_KnownSymmetric(symmetricAlgorithm);
				ITransportContextProvider iTransportContextProvider = (ITransportContextProvider) ChannelServices.GetChannel("IPMulticastListener");
				iTransportContextProvider.ITransportContext.IKeyStore.SetKey("/exchange/sk", keyProvider_KnownSymmetric);
				iTransportContextProvider = (ITransportContextProvider) ChannelServices.GetChannel("gtcp");
				iTransportContextProvider.ITransportContext.IKeyStore.SetKey("/exchange/sk", keyProvider_KnownSymmetric);

				// subscribe a listener to an event
				IExchangeEventProvider iExchangeEventProvider = (IExchangeEventProvider) Activator.GetObject(typeof(IExchangeEventProvider),
					ConfigurationSettings.AppSettings["RemoteHostUri"] + "/EventProvider.rem");
				iExchangeEventProvider.Subscribe(Client.MessageReceiver);

				// attach the listener to the multicast court
				CourtCollection.Attach("/Test/EventReceiver", Client.MessageReceiver, iExchangeEventProvider.GetAnyMbr());

				Console.WriteLine("Subscribed. Press ENTER to exit.");
				Console.ReadLine();
			}
			catch(Exception ex)
			{
				Console.WriteLine("Exception: {0}. Stack trace: {1}.", ex.Message, ex.StackTrace);
			}
		}

		public static void GenuineChannelsEventHandler(object sender, GenuineEventArgs e)
		{
			if (e.SourceException == null)
				Console.WriteLine("Global event: {0}\r\nUrl: {1}", e.EventType, 
					e.HostInformation == null ? "<not specified>" : e.HostInformation.ToString());
			else
				Console.WriteLine("Global event: {0}\r\nUrl: {1}\r\nException: {2}", e.EventType, 
					e.HostInformation == null ? "<not specified>" : e.HostInformation.ToString(), 
					e.SourceException);
		}
	}
}
