using System;
using System.Security.Cryptography;

using KnownObjects;

namespace Server
{
	/// <summary>
	/// KeyProviderImplementation.
	/// </summary>
	public class KeyProviderImplementation : MarshalByRefObject, IExchangeKeyProvider
	{
		public KeyProviderImplementation()
		{
			this.SymmetricAlgorithm = SymmetricAlgorithm.Create();
			this.SymmetricAlgorithm.Mode = CipherMode.ECB;
		}

		public SymmetricAlgorithm SymmetricAlgorithm;

		/// <summary>
		/// Returns a key for using in symmetric encryption algorithms.
		/// </summary>
		public byte[] GetKey()
		{
			return this.SymmetricAlgorithm.Key;
		}

		/// <summary>
		/// This is to insure that when created as a Singleton, the first instance never dies,
		/// regardless of the expired time.
		/// </summary>
		/// <returns></returns>
		public override object InitializeLifetimeService()
		{
			return null;
		}
	}
}
