using System;
using System.Collections;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Lifetime;
using System.Runtime.Remoting.Messaging;

using KnownObjects;
using Belikov.GenuineChannels;
using Belikov.GenuineChannels.BroadcastEngine;
using Belikov.GenuineChannels.DotNetRemotingLayer;
using Belikov.GenuineChannels.Logbook;
using Belikov.GenuineChannels.Security;
using Belikov.GenuineChannels.TransportContext;

namespace Server
{
	/// <summary>
	/// Server.
	/// </summary>
	public class Server
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			try
			{
				// setup .NET remoting
				System.Configuration.ConfigurationSettings.GetConfig("DNS");
				GenuineGlobalEventProvider.GenuineChannelsGlobalEvent += new GenuineChannelsGlobalEventHandler(GenuineChannelsEventHandler);
				RemotingConfiguration.Configure("Server.exe.config");
				//GlobalLoggerContainer.Logger = new BinaryLog(@"c:\tmp\server.log");

				KeyProviderImplementation keyProviderImplementation = new KeyProviderImplementation();
				ITransportContextProvider iTransportContextProvider = (ITransportContextProvider) ChannelServices.GetChannel("IPMulticastSender");
				iTransportContextProvider.ITransportContext.IKeyStore.SetKey("/exchange/sk", new KeyProvider_KnownSymmetric(keyProviderImplementation.SymmetricAlgorithm));
				iTransportContextProvider = (ITransportContextProvider) ChannelServices.GetChannel("gtcp");
				iTransportContextProvider.ITransportContext.IKeyStore.SetKey("/exchange/sk", new KeyProvider_KnownSymmetric(keyProviderImplementation.SymmetricAlgorithm));

				// bind the server
				RemotingServices.Marshal(new EventProviderImplementation(), "EventProvider.rem");
				RemotingServices.Marshal(keyProviderImplementation, "KeyProvider.rem");

				Console.WriteLine("Server has been started. Press enter to exit.");
				Console.ReadLine();
			}
			catch(Exception ex)
			{
				Console.WriteLine("Exception: {0}. Stack trace: {1}.", ex.Message, ex.StackTrace);
			}
		}

		/// <summary>
		/// Catches Genuine Channels events and removes client session when
		/// user disconnects.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		public static void GenuineChannelsEventHandler(object sender, GenuineEventArgs e)
		{
			if (e.SourceException == null)
				Console.WriteLine("Global event: {0}\r\nUrl: {1}", e.EventType, 
					e.HostInformation == null ? "<not specified>" : e.HostInformation.ToString());
			else
				Console.WriteLine("Global event: {0}\r\nUrl: {1}\r\nException: {2}", e.EventType, 
					e.HostInformation == null ? "<not specified>" : e.HostInformation.ToString(), 
					e.SourceException);
		}

	}
}
